using System;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Data;
using System.Globalization;
using System.Windows.Forms;

namespace DBSync.Updater
{
    public partial class UpdaterForm1 : Form
    {
        /// <summary>
        /// the connection obejct
        /// </summary>
        private SqlConnection _sqlConnection;
        private OleDbConnection _oleDbConnection;

        /// <summary>
        /// connection string to connect to sql server
        /// </summary>
        private string _connString = "Data Source=localhost;Initial Catalog=Northwind;User ID=sa";

        private int sleepInterval = 2000;
        /// <summary>
        /// thread to update the unit price in the database
        /// </summary>
        System.Threading.Thread updater;

        /// <summary>
        /// Returns the connection string of this datasource.
        /// </summary>
        public string ConnString { get { return _connString; } }

        public string dbms = "";

        public UpdaterForm1()
        {
            CheckForIllegalCrossThreadCalls = false;
            _connString = System.Configuration.ConfigurationSettings.AppSettings["conn-string"].ToString();
            dbms = System.Configuration.ConfigurationSettings.AppSettings["dbms"].ToString();
            Connect("");// connect to server
            InitializeComponent();
        }

        #region >>      === Database Connectivity ===       <<

        /// <summary>Establish connection with the datasource.
        /// </summary>
        /// <param name="connString"></param>
        public void Connect(string connString)
        {
            try
            {
                if (!dbms.Equals("other", StringComparison.OrdinalIgnoreCase))
                {
                    _sqlConnection = new SqlConnection(_connString);
                    _sqlConnection.Open();
                }
                else
                {
                    _oleDbConnection = new OleDbConnection(_connString);
                    _oleDbConnection.Open();
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                throw ex;
            }
        }

        /// <summary>Releases the connection.
        /// </summary>
        public void DisConnectDB()
        {
            if (_sqlConnection != null)
                _sqlConnection.Close();

            if (_oleDbConnection != null)
                _oleDbConnection.Close();
        }

        #endregion

        #region >>      === UI Event Handlers ===      <<
        /// <summary>start/stop button event handler.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStartStop_Click(object sender, EventArgs e)
        {
            if (updater == null)
            {
                // start the thread if it hasn't been started yet.
                updater = new System.Threading.Thread(Start);
                updater.Priority = System.Threading.ThreadPriority.BelowNormal;
                updater.Name = "updater";
                updater.Start(sleepInterval);
                btnStartStop.Text = "&Stop";

                InitializeLog();
            }
            else if(updater != null)//.ThreadState == System.Threading.ThreadState.Running)
            {
                // Interrupt the thread if it is running
                updater.Abort();
                updater = null;
                btnStartStop.Text = "&Start";
            }
        }


        private void btnDone_Click(object sender, EventArgs e)
        {
            this.Dispose(true);
        }

        #endregion
        
        #region >>      === Thread Functions ===      <<


        /// <summary>method that updater thread runs
        /// infinately unless application is closed.
        /// this thread is only suspended or resumed by the appliation.
        /// </summary>
        /// <param name="sleepInterval"></param>
        private void Start(object sleepInterval)
        {
            while (true)
            {
//                flag = false;
                Random ran = new Random();

                int id = ran.Next(1, 15);   // select one row from first 20.
                int newPrice = ran.Next(0,1500);

                UpdateProductPrice(id, newPrice); // update the price randomly
                System.Threading.Thread.Sleep((int)sleepInterval);
            }
        }

        /// <summary>updates the price in the database 
        /// randomly with random values
        /// </summary>
        /// <param name="id"></param>
        /// <param name="newPrice"></param>
        private void UpdateProductPrice(int id, int newPrice)
        {
            try
            {
                if (!dbms.Equals("other", StringComparison.OrdinalIgnoreCase))
                {
                    SqlCommand cmd = _sqlConnection.CreateCommand();
                    cmd.CommandText = String.Format(CultureInfo.InvariantCulture,
                                        "Update Products " +
                                        "Set UnitPrice = {0} " +
                                        "Where ProductID = '{1}'",
                                        newPrice, id);

                    cmd.ExecuteNonQuery();
                }
                else
                {
                    OleDbCommand cmd = _oleDbConnection.CreateCommand();
                    cmd.CommandText = String.Format(CultureInfo.InvariantCulture,
                                        "Update Products " +
                                        "Set UnitPrice = {0} " +
                                        "Where ProductID = '{1}'",
                                        newPrice, id);

                    cmd.ExecuteNonQuery();
                }
                WriteToLog(id, newPrice.ToString());
            }
            catch (Exception e)
            {
                WriteToLog(id, e.Message);
                System.Diagnostics.Trace.WriteLine(e);
            }
        }

        #endregion 

        #region  >>      === Utility Functions ===     <<

        public void InitializeLog()
        {
            this._txtLog.Text = " Product Price Update Log" + "\r\n\r\n";
            this._txtLog.AppendText(String.Format("   {0,-4}\t{1,-35} ", "ID", "Unit Price") + "\r\n");
            this._txtLog.AppendText(String.Format("  {0,-4}\t{1,-35}\r\n", "----", "-----------"));
        }
        
        //Write the result of updation in Log'
        public void WriteToLog(int productId, string value)
        {
            string entry;
            entry = String.Format("   {0,-4}\t {1,-35}\r\n", productId, value);
            this._txtLog.AppendText(entry);
            this._txtLog.Focus();
            this._txtLog.ScrollToCaret();
        }

        //Write the result of updation in Log'
        public void WriteToLog(string text)
        {
            _txtLog.Text += text;
        }

        #endregion
    }
}